//
// Handle DS2417 RTC
// By Anders Gustafsson 2017-04-17
//
#include "1wireConfig.h"
#include "1wire.h"
#include "ds2417.h"
#include <stdio.h>
#include "xprintf.h"
//
// Init a DS2417 that is the only unit on the onewire
// Start clock. Set to number of seconds in argument
//
void DS2417_init(unsigned long time)
{
    int i;
    union DS2417 {
       unsigned long time;
       unsigned char ch[4];
    } data; 
    //
    data.time = time;
    OW_reset_pulse();
    OW_write_byte (DS2417_SKIP_ROM);		// Send a command to read skip ROM
    OW_write_byte (DS2417_WRITE);
    OW_write_byte (0x0C);                   // Start clock
    for(i = 0; i<4; i++)
    {
        OW_write_byte (data.ch[i]);
    }
    //
    //The new time data is copied from the read/write buffer to the real-time
    //clock counter and becomes effective as the bus master generates a reset pulse. 
    OW_reset_pulse();
}
//
// Read time from a DS2417 that is the only unit on the onewire
// Returns number of seconds or zero if DS2417 not initialised.
//
unsigned long DS2417_read()
{
    int i;
    // Convenient construct to handle time as long or bytes
    DS2417 data; 
    //
    OW_reset_pulse();
    OW_write_byte (DS2417_SKIP_ROM);		// Send a command to read skip ROM
    OW_write_byte (DS2417_READ);
    //
    // Read status byte
    //
    data.ch[0] = OW_read_byte();
    if(data.ch[0] == 0) return 0;
    //
    for(i = 0; i<4; i++)
    {
            data.ch[i] = OW_read_byte();	// one control byte, plus 32-bit time
            //xprintf(" %X",ch);
    }
    //xprintf("%X %X %X %X %lu   \n",data.ch[0], data.ch[1], data.ch[2], data.ch[3], data.time); 
    if(data.time > 5) return data.time;    // Assume just started
    return 0;
}

/**********************************************************************
* Function:        unsigned char Detect_Slave_Device(void)
* PreCondition:    None
* Input:		   None	
* Output:		   1 - Not Present   0 - Present 	
* Overview:   	   To check the presence of slave device.    
***********************************************************************/

unsigned char Detect_Slave_Device(void)
{
	if (!OW_reset_pulse())
		return HIGH;
	else 		
		return LOW;
}
