// This is a guard condition so that contents of this file are not included
// more than once.  
//
// Structures for HP LIF-format disks
//
#ifndef LIFDISK_H
#define	LIFDISK_H

#include <xc.h> // include processor files - each processor file is guarded.  
#include <stdio.h>
#include <string.h>
#include "integer.h"
#include "Globals.h"

typedef struct {
	char unit;
	char cyl;
	char head;
	char sector;
} DiskType;

typedef struct {		// Controller description, 5 bytes
	BYTE C1;			// Installed unit byte, one unit:
	BYTE C2;			// C1 = 10000000 C2 = 00000001
	BYTE C3;			// Transfer rate in kb/s MSB
	BYTE C4;			// LSB
	BYTE C5;			// Controller type 4 = SS/80 single unit
} ControllerDescriptionType;

typedef struct {		// Unit description, 19 bytes
	BYTE U1;			// Type 0-Fixed, 1-Flexible, 2-Tape (+128-dumb, does not detect media change)
	BYTE U2;			// Device number MSB
	BYTE U3;			// (HP 9133 = 09 13 30)
	BYTE U4;			// LSB
	BYTE U5;			// Bytes per block MSB
	BYTE U6;			// LSB
	BYTE U7;			// Number of buffered blocks
	BYTE U8;			// Burst size (0 for SS/80)
	BYTE U9;			// Block time in ms MSB
	BYTE U10;			// LSB
	BYTE U11;			// Continous average transfer rate MSB
	BYTE U12;			// LSB
	BYTE U13;			// Optimal retry in tens of ms MSB
	BYTE U14;			// LSB
	BYTE U15;			// Access time in tens of ms MSB
	BYTE U16;			// LSB
	BYTE U17;			// Maximum interleave or 0
	BYTE U18;			// Fixed volume byte, one bit per volume, ie 00000111 = 3 volumes
	BYTE U19;			// Removable volume byte, one bit per volume, ie 00000111 = 3 volumes
} UnitDescriptionType;

typedef struct {		// Volume description,  bytes
	BYTE V1;			// Max cylinder MSB
	BYTE V2;			// 
	BYTE V3;			// LSB
	BYTE V4;			// Maximum head
	BYTE V5;			// Maximum sector, MSB
	BYTE V6;			// LSB
	BYTE V7;			// Max value of single vector in blocks MSB
	BYTE V8;
	BYTE V9;
	BYTE V10;
	BYTE V11;
	BYTE V12;			// LSB
	BYTE V13;			// Interleave

} VolumeDescriptionType;

typedef union {
	DWORD Length;
	BYTE Lenbytes[4];
	} LengthType;

typedef struct {
int LIFid;
char label[6];
int dirstarthi;
int dirstartlo;
int s3000;
int dummy;
int dirlenhi;
int dirlenlo;
int version;
} VolumeLabelType;


typedef struct {
	char filename[10];
	int filetype;
	int startaddhi;
	int startaddlo;
	int lengthhi;
	int lengthlo;
	char createtime[6];	// BCD digits
	int volnumber;
	int implementationhi;
	int implementationlo;
} DirEntryType;
//
// Functions
//
DWORD FindFile(FIL file, char *name);

#endif	/* LIFDISK_H */

