/*-------------------------------------------------------------------------
  sio.c - Serial in/out 

             (c) 2010 - 2017 Anders Gustafsson <anders.gustafsson@pedago.fi> 

-------------------------------------------------------------------------*/
#include <xc.h>
#include <stdio.h>

#include "sio.h"


#define PIC_CLK 64000000 //MHz
//#warning Compiled for 64.00Mhz and a baudrate of BAUD

char rx_buff[64];		// Receive data buffer
char rx_ptr;			// pointer to next slot in receive buffer
volatile union
{
	struct
	{
		unsigned DataAvailable:1;
		unsigned :7;
	};
} rx_flagsbits;
//
// http://microchipdeveloper.com/faq:31
//
//void interrupt low_priority   LowIsr(void)
//{
//	unsigned char c;
//
//	c = RCREG;
//	if(rx_ptr < 64 && c!=0x0a)			// Simplistic receive buffering. Stuff data into buffer, excluding LF
//	{									// and raise flag when we have received the entire string
//		rx_buff[rx_ptr] = c;
//		++rx_ptr;
//		//
//		
//
//	}
//	if( c == 0x0a || c == 0x0d) rx_flagsbits.DataAvailable = 1; // LF or Ctrl-C terminates message
//}
void sio_setup(void)
{
	//#define BAUD 57600
	#define BAUD 115200
    // This is for BRG16 = 0, BRGH = 0
	//#define SPBRG_VALUE ((PIC_CLK/(64UL * BAUD) -1))
	//#define BRGH_VALUE ((PIC_CLK/(64UL * BAUD) -1))>>8
    // This is for BRG16 = 0 OR BRGH = 0
	//#define SPBRG_VALUE ((PIC_CLK/(16UL * BAUD) -1))
	//#define BRGH_VALUE ((PIC_CLK/(16UL * BAUD) -1))>>8
    // This is for BRG16 = 1, BRGH = 1
	#define SPBRG_VALUE ((PIC_CLK/(4UL * BAUD) -1))
	#define BRGH_VALUE ((PIC_CLK/(4UL * BAUD) -1))>>8
    //
    BAUDCONbits.BRG16 = 1;
    TXSTAbits.BRGH = 1;
    //
    RC6PPS = 0x09;                          // RC6 pin is TX
    BAUD1CONbits.SCKP = 0;
	SPBRG=SPBRG_VALUE;
	SPBRGH=BRGH_VALUE;	//data rate for sending (see 16F876 manual under 'USART') 

	TXSTAbits.SYNC=0;						//asynchronous
	RCSTAbits.SPEN=1;						//enable serial port pins
	RCSTAbits.CREN=1;						//enable reception
	RCSTAbits.SREN=0;						//no effect
	PIE3bits.TX1IE=0;						//disable tx interrupts
	//PIE3bits.RCIE=0;						//disable rx interrupts
	INTCONbits.IPEN=1;						//enable interrupt priority
	//IPR3bits.RCIP=1;						//make receive interrupt high priority
	INTCONbits.GIEH=1;						//enable all high priority interrupts
	TXSTAbits.TX9D=0;						//8-bit transmission
	RCSTAbits.RX9=0;						//8-bit reception
	TXSTAbits.TXEN=0;						//reset transmitter
	TXSTAbits.TXEN=1;						//enable the transmitter
	//
	rx_flagsbits.DataAvailable = 0;
	rx_ptr=0;
}

//writes a character to the serial port
void sio_putch(char c)
{
	while(!PIR3bits.TX1IF)			//set when register is empty
	{
		clear_usart_errors_inline();
		ClrWdt();
	}
	TXREG=c;
}
//writes a character to the serial port
void putch(char c)
{
	while(!PIR3bits.TX1IF)			//set when register is empty
	{
		clear_usart_errors_inline();
		ClrWdt();
	}
	TXREG=c;
}
//gets a character from the serial port without timeout
//char sio_getch(void)
//{
////	while(!PIR3bits.RCIF)
////	{
////		ClrWdt();
////		clear_usart_errors_inline();
////	}
//	return RCREG;
//}
//
//char sio_getch_available(void)
//{
////  if (PIR3bits.RCIF) {
////    return true;
////  }
//  return false;
//}

void clear_usart_errors_inline(void)
{
//	unsigned char dummy;
//	if (RCSTAbits.OERR)			// Overrun error											
//	{																	
//		TXSTAbits.TXEN=0;													
//		TXSTAbits.TXEN=1;													
//		RCSTAbits.CREN=0;													
//		RCSTAbits.CREN=1;												
//	}																	
//	if (RCSTAbits.FERR)			// Framing error										
//	{																	
//		dummy=RCREG;										
//		TXSTAbits.TXEN=0;													
//		TXSTAbits.TXEN=1;													
//	}
}

void sio_puts(char *s)
{
    //sio_putch(*s);
	while(*s != 0)
	{
		sio_putch(*s);
        //sio_putch('x');
		++s;
	}
}