/*------------------------------------------------------------------------/
/  Universal string handler for user console interface
/-------------------------------------------------------------------------/
/
/  Copyright (C) 2011, ChaN, all right reserved.
/
/ * This software is a free software and there is NO WARRANTY.
/ * No restriction on use. You can use, modify and redistribute it for
/   personal, non-profit or commercial products UNDER YOUR RESPONSIBILITY.
/ * Redistributions of source code must retain the above copyright notice.
/
/-------------------------------------------------------------------------*/
// Stripped version. Output to seial only
#include "xprintf.h"
#include <xc.h>
#include "sio.h"
#if _USE_XFUNC_OUT
#include <stdarg.h>
static char *outptr;




/*----------------------------------------------*/
/* Formatted string output                      */
/*----------------------------------------------*/
/*  xprintf("%d", 1234);			"1234"
    xprintf("%6d,%3d%%", -200, 5);	"  -200,  5%"
    xprintf("%-6u", 100);			"100   "
    xprintf("%ld", 12345678L);		"12345678"
    xprintf("%04x", 0xA3);			"00a3"
    xprintf("%08LX", 0x123ABC);		"00123ABC"
    xprintf("%016b", 0x550F);		"0101010100001111"
    xprintf("%s", "String");		"String"
    xprintf("%-4s", "abc");			"abc "
    xprintf("%4s", "abc");			" abc"
    xprintf("%c", 'a');				"a"
    xprintf("%f", 10.0);            <xprintf lacks floating point support>
*/

static
void xvprintf (
	const char*	fmt,	/* Pointer to the format string */
	va_list arp			/* Pointer to arguments */
)
{
	unsigned int r, i, j, w, f;
	unsigned long v;
	char s[16], c, d, *p;


	for (;;) {
		c = *fmt++;					/* Get a char */
		if (!c) break;				/* End of format? */
		if (c != '%') {				/* Pass through it if not a % sequense */
			sio_putch(c); continue;
		}
		f = 0;
		c = *fmt++;					/* Get first char of the sequense */
		if (c == '0') {				/* Flag: '0' padded */
			f = 1; c = *fmt++;
		} else {
			if (c == '-') {			/* Flag: left justified */
				f = 2; c = *fmt++;
			}
		}
		for (w = 0; c >= '0' && c <= '9'; c = *fmt++)	/* Minimum width */
			w = w * 10 + c - '0';
		if (c == 'l' || c == 'L') {	/* Prefix: Size is long int */
			f |= 4; c = *fmt++;
		}
		if (!c) break;				/* End of format? */
		d = c;
		if (d >= 'a') d -= 0x20;
		switch (d) {				/* Type is... */
		case 'S' :					/* String */
			p = va_arg(arp, char*);
			for (j = 0; p[j]; j++) ;
			while (!(f & 2) && j++ < w) sio_putch(' ');
			sio_puts(p);
			while (j++ < w) sio_putch(' ');
			continue;
		case 'C' :					/* Character */
			sio_putch((char)va_arg(arp, int)); continue;
		case 'B' :					/* Binary */
			r = 2; break;
		case 'O' :					/* Octal */
			r = 8; break;
		case 'D' :					/* Signed decimal */
		case 'U' :					/* Unsigned decimal */
			r = 10; break;
		case 'X' :					/* Hexdecimal */
			r = 16; break;
		default:					/* Unknown type (passthrough) */
			sio_putch(c); continue;
		}

		/* Get an argument and put it in numeral */
		v = (f & 4) ? va_arg(arp, long) : ((d == 'D') ? (long)va_arg(arp, int) : (long)va_arg(arp, unsigned int));
		if (d == 'D' && (v & 0x80000000)) {
			v = 0 - v;
			f |= 8;
		}
		i = 0;
		do {
			d = (char)(v % r); v /= r;
			if (d > 9) d += (c == 'x') ? 0x27 : 0x07;
			s[i++] = d + '0';
		} while (v && i < sizeof(s));
		if (f & 8) s[i++] = '-';
		j = i; d = (f & 1) ? '0' : ' ';
		while (!(f & 2) && j++ < w) sio_putch(d);
		do sio_putch(s[--i]); while(i);
		while (j++ < w) sio_putch(' ');
	}
}


void xprintf (			/* Put a formatted string to the default device */
	const char*	fmt,	/* Pointer to the format string */
	...					/* Optional arguments */
)
{
	va_list arp;


	va_start(arp, fmt);
	xvprintf(fmt, arp);
	va_end(arp);
}


/*----------------------------------------------*/
/* Dump a line of binary dump                   */
/*----------------------------------------------*/

//void put_dump (
//	const void* buff,		/* Pointer to the array to be dumped */
//	unsigned long addr,		/* Heading address value */
//	int len,				/* Number of items to be dumped */
//	int width				/* Size of the items (DW_CHAR, DW_SHORT, DW_LONG) */
//)
//{
//	int i;
//	const unsigned char *bp;
//	const unsigned short *sp;
//	const unsigned long *lp;
//
//
//	xprintf("%08lX:", addr);		/* address */
//
//	switch (width) {
//	case DW_CHAR:
//		bp = buff;
//		for (i = 0; i < len; i++)		/* Hexdecimal dump */
//			xprintf(" %02X", bp[i]);
//		sio_putch(' ');
//		for (i = 0; i < len; i++)		/* ASCII dump */
//			sio_putch((bp[i] >= ' ' && bp[i] <= '~') ? bp[i] : '.');
//		break;
//	case DW_SHORT:
//		sp = buff;
//		do								/* Hexdecimal dump */
//			xprintf(" %04X", *sp++);
//		while (--len);
//		break;
//	case DW_LONG:
//		lp = buff;
//		do								/* Hexdecimal dump */
//			xprintf(" %08LX", *lp++);
//		while (--len);
//		break;
//	}
//
//	sio_putch('\n');
//}

#endif /* _USE_XFUNC_OUT */



